@@ -34,11 +34,7 @@
 #include "IIOP.h"
 #include "IIOP-private.h"
 
-#ifdef HAVE_LIMITED_WRITEV
-#define writev g_writev
-#endif
-
-/* type defs */
+#define MAX_LIMITED_IOVECS 16
 
 #ifdef        __GNUC__
 #define       PACKED __attribute__((packed))
@@ -172,76 +168,92 @@ giop_send_buffer_new(void)
 gint
 giop_send_buffer_write(GIOPSendBuffer *send_buffer)
 {
-  gulong nvecs;
-  glong res, sum, t;
-  struct iovec *curvec;
-  int fd;
-  GIOPConnection *cnx;
-  gint retval = -1;
+	gulong nvecs;
+	glong res, length_to_write;
+	struct iovec *curvec;
+	int fd;
+	gboolean set_blocking;
+	GIOPConnection *cnx;
+	gint retval = -1;
 
-  cnx = GIOP_MESSAGE_BUFFER(send_buffer)->connection;
-  if(!cnx->is_valid)
-    return -1;
+	cnx = GIOP_MESSAGE_BUFFER (send_buffer)->connection;
+	if (!cnx->is_valid)
+		return -1;
 
-  fd = GIOP_CONNECTION_GET_FD(cnx);
-  nvecs = GIOP_MESSAGE_BUFFER(send_buffer)->iovecs->len;
-  curvec = (struct iovec *)GIOP_MESSAGE_BUFFER(send_buffer)->iovecs->data;
+	fd = GIOP_CONNECTION_GET_FD (cnx);
+	nvecs = GIOP_MESSAGE_BUFFER (send_buffer)->iovecs->len;
+	curvec = (struct iovec *) GIOP_MESSAGE_BUFFER (send_buffer)->iovecs->data;
 
-#if defined(ORBIT_DEBUG) && 0
-  g_print("Message of length %d looks like:\n",
-	  GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_size);
-  for(sum = 0, t = 2; t < nvecs; t++) {
+#if defined (ORBIT_DEBUG) && 0
+	{
+		gulong sum, t;
 
-    sum += curvec[t].iov_len;
-    g_print("    [%p, %d]: %d\n", curvec[t].iov_base, curvec[t].iov_len,
-	    sum);
-  }
+		g_print ("Message of length %d looks like:\n",
+			 GIOP_MESSAGE_BUFFER (send_buffer)->message_header.message_size);
+		for (sum = 0, t = 0; t < nvecs; t++) {
+			sum += curvec[t].iov_len;
+			g_print ("    [%p, %d]: cumul. %ld   - vec idx %d\n",
+				 curvec[t].iov_base,
+				 curvec[t].iov_len, sum, t);
+		}
+	}
 #endif
-  res = writev(fd, curvec, nvecs);
+	length_to_write = sizeof (GIOPMessageHeader)+ 
+		GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_size;
+	set_blocking = FALSE;
+	do {
+  
+		do {
+#ifdef HAVE_LIMITED_WRITEV
+/*			fprintf (stderr, "ltw %ld write %d vecs, first of length %d ",
+			length_to_write, nvecs, (int) curvec->iov_len);*/
+			res = writev (fd, curvec, MIN (nvecs, MAX_LIMITED_IOVECS));
+/*			fprintf (stderr, "wrote %ld bytes [%d]\n", res, (int) errno); */
+#else
+			res = writev (fd, curvec, nvecs);
+#endif
+		} while (res < 0 && errno == EINTR);
 
-  sum = (GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_size + sizeof(GIOPMessageHeader));
-  if(res < sum) {
-    if(res < 0) {
-      if(errno != EAGAIN) {
-	giop_main_handle_connection_exception(cnx);
-	goto out;
-      }
+		if (res < 0) {
+			if (errno != EAGAIN) {
+				giop_main_handle_connection_exception (cnx);
+				goto out;
+			} else
+				res = 0;
+		}
 
-      res = 0;
-    }
+		length_to_write -= res;
 
-    /* wrote 7, iovecs 3, 2, 2, 4:
-       0 + 3 !> 7
-       3 + 2 !> 7
-       5 + 2 !> 7
-     */
+		/* Move along the vector array */
+		if (length_to_write > 0) {
+			gulong t;
 
-    for(t = 0; ; t += curvec->iov_len, curvec++, nvecs--) {
-      if((t + curvec->iov_len) > res)
-	break;
-    }
-    if((res - t) > 0) {
-      curvec->iov_len -= (res - t);
-      curvec->iov_base = (gpointer)((char *)curvec->iov_base + (res - t));
-    }
+			if (!set_blocking) {
+				set_blocking = TRUE;
+				fcntl (fd, F_SETFL, fcntl (fd, F_GETFL, 0) & ~O_NONBLOCK);
+			}
 
-    fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) & ~O_NONBLOCK);
+			for (t = 0; (t + curvec->iov_len) <= res; ) {
+				t += curvec->iov_len;
+				curvec++, nvecs--;
+			}
+			/* t is now the count of the bytes in completely written iovecs */
 
-    t = writev(fd, curvec, nvecs);
+			if ((res - t) > 0) {
+				curvec->iov_len -= (res - t);
+				curvec->iov_base = ((guchar *)curvec->iov_base + (res - t));
+			}
+		}
 
-    fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) | O_NONBLOCK);
+	} while (length_to_write > 0);
 
-    if((t < 0) || ((res + t) < sum)) {
-	giop_main_handle_connection_exception(cnx);
-	goto out;
-    }
-  }
+	if (set_blocking)
+		fcntl (fd, F_SETFL, fcntl (fd, F_GETFL, 0) | O_NONBLOCK);
 
-  retval = 0;
+	retval = 0;
 
  out:
-
-  return retval;
+	return retval;
 }
 
 static GIOPSendBuffer *
@@ -913,7 +925,7 @@ giop_recv_message_buffer_use(GIOPConnection *connection)
 		  retval->left_to_read);
 
     if((sysret == 0)
-       || ((sysret < 0) && (errno != EAGAIN)))
+       || ((sysret < 0) && (errno != EAGAIN) && (errno != EINTR)))
       goto errout;
 
     if(sysret > 0)
